<?php

namespace ID3\Bases;

class AdminInstance extends Plugin{

    public $scriptsRendered = false;
    
    function __construct($namespace){
        parent::__construct($namespace);
        

        $this->registerHooks([
            'admin_menu' => null,
            'network_admin_menu' => null,
            'admin_enqueue_scripts'=>'load_resources',
            'id3/framework/render_scripts' => 'renderScripts'
        ]);
    }

    function getCamelCaseName(){
        $ccName =  lcfirst(str_replace(' ','', ucwords( str_replace('-',' ',$this->name) ) ));
        
        return $ccName;
    }

    function getControllerName(){
        $key = str_replace('id3','',$this->key);

        $name = lcfirst(str_replace(' ','', ucwords( str_replace('-',' ',$key) ) ));
        return $name . 'Ctrl';
    }
    function getNetworkControllerName(){
        $key = str_replace('id3','',$this->key);

        $name = lcfirst(str_replace(' ','', ucwords( str_replace('-',' ',$key) ) ));
        return $name . 'NetworkCtrl';
    }

    function registerHooks($actions){
        foreach ($actions as $key => $value) {
            $value = ($value == null) 
                ? 'register' . str_replace(' ','', ucwords( str_replace('_',' ',$key) ) ) // turn foo_bar into registerFooBar
                : $value;


            if(method_exists($this, $value)){
                add_action($key, [$this, $value]);
            }
        }
    }

    function registerAdminMenu(){
        //\ID3\Helpers\add_admin_menu_separator(30);
        
        $this->registerSubMenu();
        
    }

    function registerNetworkAdminMenu(){
        
        $this->registerNetworkSubMenu();
    }

    function registerSubMenu(){
        if(!isset($this->name)) return;

        add_submenu_page(
            'id3-tools'
            , $this->name
            , $this->name
            , 'manage_options' 
            , 'id3-' . sanitize_title($this->name)
            , [$this, 'renderAdminPage']
        );

        
    }

    function registerNetworkSubMenu(){
        if(!isset($this->name)) return;
        if(!$this->hasNetworkConfigs()) return;

        add_submenu_page(
            'id3-tools'
            , $this->name
            , $this->name
            , 'manage_options' 
            , 'id3-' . sanitize_title($this->name)
            , [$this, 'renderNetworkAdminPage']
        );
    }


    function load_resources($page){
        $pluginKey = $this->key;
        $ressource_location = 'id-3_page_' . $this->key;

        if($this->key == 'id3-framework') $ressource_location = 'toplevel_page_id3-tools';
        
        if($page == $ressource_location){
            wp_enqueue_media();
            wp_enqueue_style('ang-material', 'https://ajax.googleapis.com/ajax/libs/angular_material/1.1.8/angular-material.min.css');
            wp_enqueue_style('animage-css', 'https://cdnjs.cloudflare.com/ajax/libs/animate.css/4.1.1/animate.min.css');
            wp_enqueue_style( 'fontawesome5', '/' . \ID3\Framework\PLUGIN_URL . 'styles/fa/css/all.min.css' );
            wp_enqueue_style( 'id3-eco-framework-style', '/' . \ID3\Framework\PLUGIN_URL. 'styles/global-admin.min.css',[], \ID3\Framework\VERSION );
            wp_enqueue_style( $this->key . '-style', plugins_url('/styles/admin.min.css', $this->root),[],$this->version );
        }        
    }

    function renderScripts(){
        
        
        $lTwoLetterLang = substr(get_locale(),0,2);
        $lPluginContext = [
            'base_url'  => $this->url,
            'key'       => $this->key,
            'version'   => $this->version,
            'rest_root' => esc_url_raw( rest_url() ), 
            'nonce'     => wp_create_nonce( 'wp_rest' ),
            'is_multisite' => is_multisite(),
            'is_network_admin' => is_network_admin()
        ];
        $lFrameworkContext = [
            'base_url'  => \ID3\Framework\PLUGIN_URL
        ];
        $onLoadActions = [];

        $scripts = [
            \ID3\Framework\PLUGIN_URL . 'scripts/ang.admin-app.min.js?v=' . filemtime(\ID3\Framework\PLUGIN_DIR . 'scripts/ang.admin-app.min.js'),
        ];
        

        if($this->url != \ID3\Framework\PLUGIN_URL){
            $scripts[] = $this->url . 'scripts/ang.admin-app.min.js?v=' . filemtime($this->dir . 'scripts/ang.admin-app.min.js');
        }
        
        if($lTwoLetterLang != 'en'){
            $onLoadActions[] = '$locales.init("' . $lTwoLetterLang . '")';

            if(file_exists(\ID3\Framework\PLUGIN_DIR . "scripts/locales/framework.{$lTwoLetterLang}.min.js")){
                $scripts[] = \ID3\Framework\PLUGIN_URL . "scripts/locales/framework.{$lTwoLetterLang}.min.js?v=" . filemtime(\ID3\Framework\PLUGIN_DIR . 'scripts/locales/framework.fr.min.js');
            }
            
            if(file_exists($this->dir . "scripts/locales/{$lTwoLetterLang}.min.js")){
                $scripts[] = $this->url . "scripts/locales/framework.{$lTwoLetterLang}.min.js?v=" . filemtime($this->dir . 'scripts/locales/framework.fr.min.js');
            }
            else if(file_exists($this->dir . "scripts/locales/{$lTwoLetterLang}.json")){
                //$onLoadActions[] = '$locales.load("/' . $this->url . "scripts/locales/{$lTwoLetterLang}.json" . '")';
                $onLoadActions[] = '$locales.add("' . $lTwoLetterLang . '",' . file_get_contents($this->dir . "scripts/locales/{$lTwoLetterLang}.json") . ')';
            }

        }

        
        foreach ($scripts as $url) {
            echo('<script src="/'. $url . '"></script>');
        }

        echo('<script type="text/javascript" id="id3-framework-script">');
        echo('const $id3PluginContext = ' . json_encode(apply_filters('id3/admin/pluginContext',$lPluginContext)) . ';');
        echo('const $id3FrameworkContext = ' . json_encode($lFrameworkContext) . ';');
        echo(implode(';', $onLoadActions));
        echo('</script>');

        
    }

    function hasMainConfigs(){
        return file_exists($this->dir . '/views/statics/main.html');
    }
    function renderMainConfigs(){
        include $this->dir . '/views/statics/main.html';
    }

    
    function renderAdminPage(){
        $lDir = \ID3\Framework\PLUGIN_DIR;
        include $lDir . '/views/page.admin.php';
    }

    function hasNetworkConfigs(){
        return file_exists($this->dir . '/views/statics/network.html');
    }
    function renderNetworkConfigs(){
        include $this->dir . '/views/statics/network.html';
    }

    function renderNetworkAdminPage(){
        $lDir = \ID3\Framework\PLUGIN_DIR;
        include $lDir . '/views/page.network-admin.php';
    }
    
}