<?php
namespace ID3\Bases;
class Block {
    private $_namespace = __NAMESPACE__;
    private $key = '';
    private $dir = '';
    private $url = '';
    private $plugin_name = '';

    public $settings = null;
    public $networkSettings = null;

    
    private $editor_file_path = '';
    private $editor_file_url = '';


    function __construct($namespace=null){
        if($namespace != null) $this->_namespace = $namespace;

        $this->key                 = constant($namespace . '\KEY');
        $this->dir                 = constant($namespace . '\PLUGIN_DIR');
        $this->url                 = constant($namespace . '\PLUGIN_URL');
        $this->plugin_name         = constant($namespace . '\NAME');

        $this->editor_file_path    = $this->dir . 'scripts/blocks.min.js';
        $this->editor_file_url     = '/' . $this->url . 'scripts/blocks.min.js';

        $this->registerActions();
    }

    function loadSettings(){
        if($this->settings != null) return;
        
        $this->settings = \ID3\Settings::get($this->key);
        if($this->settings == null){
            $this->settings = \ID3\Settings::getNetwork($this->key);
            if(is_array($this->settings)) $this->settings = json_decode(json_encode($this->settings));
        }
        else{
            $this->networkSettings = \ID3\Settings::getNetwork($this->key);
        }
        return;
    }

    function registerActions(){
        add_action( 'enqueue_block_editor_assets', [$this, 'registerEditorScripts'] );
        //add_action('init', [$this,'registerBlockEditor']);
        add_filter( 'block_categories_all', [$this, 'addCategory'], 10, 2);
    }

    function addCategory($categories, $post){
        $namespace = $this->_namespace;

        $blockCategory = array(
            'slug' => strtolower(str_replace('\\','-', $namespace)),
            'title' => $this->plugin_name,
        );
    
        array_unshift($categories, $blockCategory);
        
        return $categories;
    }

    function registerBlockEditor(){
        if ( ! function_exists( 'register_block_type' ) ) return;
        
        $blockJsonDeclarationPath = $this->dir . "scripts/blocks/block.json";

        if(file_exists($blockJsonDeclarationPath)){
            //echo($this->dir . "blocks/block.json");
            register_block_type( $this->dir . 'scripts/blocks');
        }
        else{
            $this->registerEditorScripts();
        }
    }
    
    function registerEditorScripts(){
        if(file_exists($this->editor_file_path)){
            // Existance of blocks.min.js means supports for Gutenberg Block

            

            // Enqueue the bundled block JS file
            wp_enqueue_script(
                $this->plugin_name . '-blocks-js',
                $this->editor_file_url,
                [ 'wp-i18n', 'wp-element', 'wp-blocks', 'wp-components', 'wp-editor','id3fw-blocks-js' ],
                filemtime( $this->editor_file_path )
            );
        

            

        }
        
        // Make paths variables so we don't write em twice ;)
        $editorStylePath = '/' . $this->url . 'styles/public.min.css';
        // Enqueue optional editor only styles
        wp_enqueue_style(
            $this->plugin_name . '-blocks-editor-css',
            $editorStylePath,
            []
        );
        
    }

}