<?php

namespace ID3;

// Framework Rest API
class Rest{
    private $_routes = [];

    private static $_instance = null;
    public static function instance(){
        if(self::$_instance == null){
            self::$_instance = new Rest();
        }
        return self::$_instance;
    }

    public function init() {
        $this->registerRoutes();

        
    }

    function registerRoutes(){
        //$this->addRoute('/infos/plugins', [$this, 'getPluginList']);
        $getSettingsOptions = [
            'args' => [
                'plugin' => [
                    'required' => true,
                    'type' => 'string',
                    'description' => 'Plugin name',
                ],
                'site_id' => [
                    'required' => false,
                    'type' => 'integer',
                    'description' => 'Setting site ID',
                ]
            ]
        ];
        $setSettingsOptions = [
            'methods' => \WP_REST_Server::EDITABLE,
            'args' => [
                'plugin' => [
                    'required' => true,
                    'type' => 'string',
                    'description' => 'Plugin name',
                ],
                'data' => [
                    'required' => true,
                    'type' => 'object',
                    'description' => 'Settings to save',
                ]
            ]
        ];

        $this->addRoute('/settings/get', [$this, 'getSettings'],$getSettingsOptions);
        $this->addRoute('/settings/save', [$this, 'setSettings'],$setSettingsOptions);

        $this->addRoute('/settings/getNetwork', [$this, 'getNetworkSettings'],$getSettingsOptions);
        $this->addRoute('/settings/saveNetwork', [$this, 'setNetworkSettings'],$setSettingsOptions);
        
        $this->addRoute('/themes', [$this, 'getThemeList']);

        $this->addRoute('/pages', [$this, 'getPageList']);

        $this->addRoute('/pages/get', [$this, 'getPage'],[
            'args' => [
                'id' => [
                    'required' => true,
                    'type' => 'string',
                    'description' => 'Page id',
                ]
            ]
        ]);
        $this->addRoute('/pages/search', [$this, 'searchPages'],[
            'methods' => \WP_REST_Server::CREATABLE ,
            'args' => [
                'keyword' => [
                    'required' => true,
                    'type' => 'string',
                    'description' => 'Search keywords',
                ]
            ]
        ]);


        foreach ($this->_routes as $path => $routeDef) {
            register_rest_route( 'id3-rest',$path, $routeDef);
        }
    }

    function getList(){
        return $this->_routes;
    }

    function addRoute($path, $callback, $options=null){
        $routeDef = [
            
                'methods' => \WP_REST_Server::READABLE,
                'permission_callback' => '__return_true',
                'callback' => $callback,
                'args' => []
            
        ];

        if($options != null){
            if (isset($options['methods'])){
                $routeDef['methods'] = $options['methods'];
            }
            
            if (isset($options['args'])){
                $routeDef['args'] = $options['args'];
            }
        } 
        $this->_routes[$path] = $routeDef; //register_rest_route( 'id3-rest',$path, $routeDef);
    }
    

    // global endpoints
    function getNetworkSettings($request){
        $plugin = $request->get_param('plugin');

        return \ID3\Settings::getNetwork($plugin);
    }

    function setNetworkSettings($request){
        $plugin = $request->get_param('plugin');
        $data   = $request->get_param('data');

        \ID3\Settings::setNetwork($plugin,$data);
    }

    function getSettings($request){
        $plugin = $request->get_param('plugin');
        $site_id = $request->get_param('site_id');

        if($site_id != null && $site_id != get_current_blog_id()){
            switch_to_blog($site_id);
        }

        return \ID3\Settings::get($plugin);
    }

    function setSettings($request){
        $plugin = $request->get_param('plugin');
        $data   = $request->get_param('data');

        return \ID3\Settings::set($plugin,$data);
    }

    function getThemeList($request){
        
        $themes = wp_get_themes(['allowed' => 'network','blog_id' => 1]);
        return array_values(array_map(function($elm){
            return [
                'name' => $elm->name,
                'title' => $elm->title,
                'version' => $elm->version,
                'parent_theme' => $elm->parent_theme,
                'template_dir' => $elm->template_dir,
                'stylesheet_dir' => $elm->stylesheet_dir,
                'template' => $elm->template,
                'stylesheet' => $elm->stylesheet,
                'screenshot' => ($elm->screenshot != '') ? str_replace(ABSPATH,'/',$elm->stylesheet_dir) . '/' . $elm->screenshot : '',
                'description' => $elm->description,
                'author' => $elm->author,
                'tags' => $elm->tags,
                'theme_root_uri' => $elm->theme_root_uri
            ];
        },$themes));
    }

    function getPageList($request){
        
        return get_pages();
    }

    function getPage($request){
        $id = $request->get_param('id');
        
        return get_post($id);
    }
    
    function searchPages($request){
        $kwd = $request->get_param('keyword');
        
        return get_post($id);
    }


}


namespace ID3\Bases;
class Rest {
    private $_namespace = __NAMESPACE__;

    function __construct($namespace=null){
        if($namespace != null) $this->_namespace = $namespace;
    }

    function addRoute($path, $callback, $options=null){
        $lNamespace = strtolower($this->_namespace);
        if(strpos($lNamespace,'id3\\')!== false){
            $lNamespace = substr($lNamespace,4);
        }

        if($options!=null && (isset($options['methods']))){
            if(!is_array($options['methods'])) $options['methods'] = [$options['methods']];
            if(!in_array('GET',$options['methods']) && !isset($options['permission_callback']) && (!isset($options['public']) || $options['public'] === true) ){
                $options['permission_callback'] = function(){
                    $request = $_SERVER['REQUEST_URI'];
                    if($request != '/wp-json/si-rest/configs') return false;
                    return current_user_can('edit_posts');
                };
            }
        }


        \ID3\Rest::instance()->addRoute($lNamespace . '/' . ltrim($path,'/'), $callback,$options);
    }

    function addRouteGET($path, $callback, $options=null){
        if($options === null) $options = [];
        $this->addRoute($path,$callback, array_merge(['methods' => \WP_REST_Server::READABLE], $options));
    }

    function addRoutePOST($path, $callback, $options=null){
        if($options === null) $options = [];
        $this->addRoute($path,$callback, array_merge(['methods' => \WP_REST_Server::CREATABLE], $options));
    }
    function addRouteDELETE($path, $callback, $options=null){
        if($options === null) $options = [];
        $this->addRoute($path,$callback, array_merge(['methods' => \WP_REST_Server::DELETABLE], $options));
    }
    function addRoutePUT($path, $callback, $options=null){
        if($options === null) $options = [];
        $this->addRoute($path,$callback, array_merge(['methods' => \WP_REST_Server::EDITABLE], $options));
    }

    function newRouteParam(&$options, $name, $required=false, $type="string", $description=''){
        if(!isset($options['args'])){
            $options['args'] = [];
        }
        $options['args'][$name] = [
            'required' => $required,
            'type'      => $type,
            'description'   => ($description == '') ? 'Value for ' . $name : $description
        ];

        return $options;
    }

}
