<?php
namespace ID3;

use ReflectionMethod;

use function ID3\Helpers\id3_parse_doc_comment;

class ShortcodeEngine {
    public $list = [];

    private static $_instance = null;
    public static function instance(){
        if(self::$_instance == null){
            self::$_instance = new ShortcodeEngine();
        }
        return self::$_instance;
    }

    public function init() {
        foreach ($this->list as $sc) {
            add_shortcode($sc['tag'], $sc['func']);
        }
    }

    
    function getList(){
        return $this->list;
    }

    public function add($tag, $func, $ns=null){
        if($ns != null) $tag = strtolower(str_replace('\\','_',$ns)) . '_' . $tag;
        $infos = new ReflectionMethod($func[0], $func[1]);
        $fnInfos = id3_parse_doc_comment($infos->getDocComment());


        $this->list[] = [
            'tag' => $tag,
            'func' => $func,
            'namespace' => $ns,
            'infos' => $fnInfos
        ];
    }

}


namespace ID3\Bases;

class Shortcode {
    private $_namespace = null;
    private $_settings = null;

    function __construct($namespace){
        $this->_namespace = $namespace;
    }

    function getSettings(){
        if($this->_settings == null){
            
            $settings = \ID3\Settings::get(constant($this->_namespace . '\KEY'));
            
            if($settings !== false){
                $this->_settings = json_decode(json_encode($settings));// json_decode($settings);
            }
        }

        return $this->_settings;
    }

    function registerAll($list){
        foreach ($list as $key => $value){
            $funcRef = $key;
            if(isset($list[$key])){
                $funcRef = $list[$key];
            }

            if(!is_array($funcRef)){
                $funcRef = [$this, $funcRef];
            }

            $this->register($key, $funcRef, $this->_namespace);
        }
    }

    function register($name, $func, $ns){

        \ID3\ShortcodeEngine::instance()->add($name, $func,$ns);
        
    }

}