<?php
namespace ID3\Framework;

/**
 * Plugin Rest API 
 * @package ID3\PluginSample
 * Use this class to add REST call ability to your plugin
 * For more infos, see: "Building your REST API"
 */
class Rest extends \ID3\Bases\Rest{

    function __construct(){
        parent::__construct(__NAMESPACE__);

        // Add rest route to be able to call them from the back/front-end via ajax
        $this->addRoute('/infos/plugins', [$this, 'getPluginList']);
        $this->addRoute('/infos/shortcodes', [$this, 'getShortcodeList']);
        $this->addRoute('/infos/rests', [$this, 'getRestList']);
    }

    // GET: /wp-json/id3-rest/framework/infos/plugins
    function getPluginList(){
        $activePlugins = get_option('active_plugins');
        if(is_multisite()){
            
            $activePlugins = array_merge($activePlugins, array_keys(get_site_option( 'active_sitewide_plugins' )));
        }
        //return $activePlugins;

        $ecosystemPlugins = array_map(
            function($e){
                
                $key  = substr($e, 0, strpos($e, '/'));
                $path = '/wp-content/plugins/' . $key . '/';
                $namespace = include(ABSPATH . $path . 'index.php');
                if(!defined($namespace . '\NAME')) return $namespace;

                return [
                    'path' => $path,
                    'key'  => $key,
                    'color' => constant($namespace . '\COLOR'),
                    'name' => constant($namespace . '\NAME'),
                ];
            },
            array_values(array_filter($activePlugins, function($e){
                if(strpos($e, 'id3-') === false) return false;
                if(strpos($e, 'id3-delivery-service') !== false) return false;
                if(strpos($e, 'id3-framework') !== false) return false;
                return true;
            }))
        );

        return array_values($ecosystemPlugins);
    }

    function getShortcodeList(){
        return \ID3\ShortcodeEngine::instance()->getList();
    }

    function getRestList(){
        return \ID3\Rest::instance()->getList();
    }
}