<?php

namespace ID3;

class Settings{

    static function get($plugin_key, $default_values=[]){
        $data = get_option($plugin_key . '_settings');

        if($data === false) return json_decode(json_encode($default_values));
        $result = json_decode($data);
        foreach ($default_values as $key => $value) {
            if(!isset($result->{$key})){
                $result->{$key} = $value;
            }
        }
        return $result;
    }

    static function set($plugin_key, $data){

        if(!isset($plugin_key) || empty($plugin_key)) return;
        if($data==null) return;
        

        $encodedPluginData = json_encode($data);
        if($encodedPluginData == null) return;
        if(empty($encodedPluginData)) return;

        
        $result = [
            'success' => update_option($plugin_key . '_settings', $encodedPluginData),
            'actions' => [
                'id3/settings/save',
                "id3/{$plugin_key}/settings/save"
            ]
        ];

        do_action('id3/settings/save');
        do_action("id3/{$plugin_key}/settings/save",  json_decode($encodedPluginData) );

        return $result;
    }

    static function getNetwork($plugin_key, $default_values=[]){
        $nid = get_current_network_id();
        $data= get_network_option($nid, $plugin_key . '_settings');

        if($data === false) return json_decode(json_encode($default_values));
        $result = json_decode($data);
        foreach ($default_values as $key => $value) {
            if(!isset($result->{$key})){
                $result->{$key} = $value;
            }
        }
        return $result;
    
    }

    static function setNetwork($plugin_key, $data=null){
        $nid = get_current_network_id();

        if(!isset($plugin_key) || empty($plugin_key)) return;
        if($data==null) return;
        

        $encodedPluginData = json_encode($data);
        if($encodedPluginData == null) return;
        if(empty($encodedPluginData)) return;


        update_network_option($nid, $plugin_key . '_settings', $encodedPluginData);

        do_action('id3/network/settings/save');
        do_action("id3/{$plugin_key}/network/settings/save", json_decode($encodedPluginData));
    }
}