<?php


namespace ID3\Helpers;

if(!function_exists('remote_api_call')){
    function remote_api_call($url, $data=null, $method="GET"){
        $curl = curl_init();
        switch ($method){
            case "POST":
                curl_setopt($curl, CURLOPT_POST, 1);
                if ($data)
                    curl_setopt($curl, CURLOPT_POSTFIELDS, $data);
                break;
            case "PUT":
                curl_setopt($curl, CURLOPT_CUSTOMREQUEST, "PUT");
                if ($data)
                    curl_setopt($curl, CURLOPT_POSTFIELDS, $data);			 					
                break;
            default:
                if ($data)
                    $url = sprintf("%s?%s", $url, http_build_query($data));
        }
        // OPTIONS:
        curl_setopt($curl, CURLOPT_URL, $url);
        
        curl_setopt($curl, CURLOPT_HTTPHEADER, array(
            'Content-Type: application/json'
        ));

        curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($curl, CURLOPT_HTTPAUTH, CURLAUTH_BASIC);
        
        // EXECUTE:
        $result = curl_exec($curl);
        if(!$result){
            $result = [
                'error' => curl_error($curl)
            ];
        }
        curl_close($curl);
        return $result;
    }
}


if(!function_exists('add_admin_menu_separator')){
    function add_admin_menu_separator($position) {
        global $menu;
        $index = 0;
        foreach($menu as $offset => $section) {
            if (substr($section[2],0,9)=='separator')
                $index++;
            if ($offset>=$position) {
                $menu[$position] = array('','read',"separator{$index}",'','wp-menu-separator');
                break;
            }
        }
        ksort( $menu );
    }
}

if(!function_exists('apply_plugins_dependency_order')){

    function apply_plugins_dependency_order($namespace, $priority){
        
        return;
        
        add_action('activated_plugin', function() {
            $active_plugins = get_option('active_plugins');
            

            $wp_path_to_plugin = preg_replace('/(.*)plugins\/(.*)$/', WP_PLUGIN_DIR."/$2", constant('ID3\Framework\PLUGIN'));
            
            $this_plugin = plugin_basename(trim($wp_path_to_plugin));
            
            $this_plugin_key = array_search($this_plugin, $active_plugins);
            array_splice($active_plugins, $this_plugin_key, 1);
            //array_push($active_plugins, $this_plugin);
            array_unshift($active_plugins, $this_plugin);
            
            update_option('active_plugins', $active_plugins);
            
        });

        $active_plugins = get_option('active_plugins');
        \ID3\debug($active_plugins);
    }

}

if(!function_exists('remove_class_action')){
    
    function remove_class_filter( $tag, $class, $method ){
        $filters = $GLOBALS['wp_filter'][ $tag ];

        if ( empty ( $filters ) )
        {
            return;
        }

        $removed_filters = [];

        foreach ( $filters as $priority => $filter )
        {
            foreach ( $filter as $identifier => $function )
            {
                if ( is_array( $function)
                    and is_a( $function['function'][0], $class )
                    and $method === $function['function'][1]
                )
                {
                    $removed_filters[] = [
                        'tag' => $tag,
                        'method' => array ( $function['function'][0], $method ),
                        'priority' => $priority
                    ];

                    remove_filter(
                        $tag,
                        array ( $function['function'][0], $method ),
                        $priority
                    );
                }
            }
        }

        return $removed_filters;
    }

    function restore_class_filters($removed_filters){

        if(!is_array($removed_filters)) return;

        foreach ($removed_filters as $filter) {
            add_filter(
                $filter['tag'],
                $filter['method'],
                $filter['priority']
            );
        }

    }
}

function id3_parse_doc_comment($raw=null){
    if($raw == null) return null;
    if($raw === false) return null;

    $raw = trim($raw," \t\n\r/*");

    $lDocParts = array_map(
        function($part){
            $cleanPart = trim($part, " \t*");
            return $cleanPart;
        }, 
        explode("\n",$raw)
    );

    $desc = array_filter($lDocParts, function($part){
        if(trim($part) === '') return false;
        return strpos($part, '@')!==0;
    });
    $params = array_values(array_map(
                function($part){
                    $paramParts = explode(' ', trim(str_replace('@param','',$part)));
                    $param = [
                        'type' => $paramParts[0],
                        'name' => str_replace('$','',$paramParts[1])
                    ];
                    if(count($paramParts)>2){
                        $param['description'] = implode(' ',array_slice($paramParts,2));
                    }

                    return $param;
                },
                array_filter($lDocParts, function($part){
                    if(trim($part) === '') return false;
                    return strpos($part, '@param')===0;
                })
            ));
    
    $lResult = [
        'description' => $desc,
        'args' => $params,
        '_raw' => $raw
    ];

    return $lResult;
}