id3Plugin
.controller('rootController', 
function(
    $scope, $rootScope, $element,
    $q, $timeout, 
    $id3UI,$id3Rest,$id3Settings,$id3Lists,$id3Tools){

    $scope._is_ready = false;
    $scope._current_module = null;
    $scope._auto_save = true;

    $scope.$id3Lists = $id3Lists;

    $scope.rootInit = function(){
    
        console.log('id3Plugin@rootController/rootInit');
    }

    $rootScope.isReady = function($options=null){
        $options = angular.merge({
            moduleName : null,
            autoSaveOnChange: null,
            serviceList: [$id3UI, $id3Rest,$id3Settings]
        },$options);

        $scope._current_module = $options.moduleName;
        $scope._auto_save = $options.autoSaveOnChange;
        if($scope._auto_save) $id3Settings.saveOnChange($scope._auto_save);

        if($scope._is_ready) return $q.resolve();
        return $q( ($resolve, $reject) => {
            $q.all($options.serviceList.map($s => {
                return $s.isReady()
            }))
            .then(
                _ => {
                    $
                    $resolve();
                },
                _ => {
                    $reject();
                }
            )
            .catch($err => {
                $reject($err);
            });
        });
    }

    $rootScope.getPageClass = function(){
        const lResult = [];
        if($scope._current_module != null) lResult.push($scope._current_module);
        if(id3IsNullOrEmpty($scope._auto_save) || false === $scope._auto_save) lResult.push('id3-show-edit-tools');
        
        return lResult.join(' ');
    }


    $rootScope.saveSettings = function(){
        $id3Settings.save();
    }

    $rootScope.resetSettings = function(){
        $id3UI.confirm('Are you sure you want to reset this plugin\'s settings?').then( _ => {
            $id3Settings.reset();
        });
    }

    $rootScope.deletePlugin = function(){
        $id3UI.confirm('Are you sure you want to delete this plugin?').then( _ => {
            $id3UI.alert('This feature is not yet supported', 'You will be redirected to the plugin page instead').then(_ => {
                window.location = '/wp-admin/plugins.php';
            });
        });
    }

    $rootScope.goHome = function(){
        window.location = '/wp-admin/admin.php?page=id3-tools';
    }

});

id3Plugin
.controller('dialogController', function($scope, $rootScope, $q, $timeout, $mdDialog, $id3Lists, $params){
    $scope.dialogOptions = {
        actions : [
            {label: 'OK', action: function(){ $scope.close(); }, primary: true}
        ]
    };

    $scope.$id3Lists = $id3Lists;

    $scope.cancel = function(){
        $scope.close();
    }
    $scope.close = function(){
        $mdDialog.cancel();
    }

    $scope.closeAndReturn = function($value){
        $mdDialog.hide($value);
    }

    $scope.$params = $params;
    
    $scope.dialogInit = function(){

    }

    $scope.trapDialogKeydown = function($event){
        if($scope.dialogOptions.actions.length == 0) return;
        console.log('dialogController/trapDialogKeydown');
        const lKeyCode = $event.keyCode;
        const lKeyMap = {
            13 : _ => {
                const lPrimary = $scope.dialogOptions.actions.find($a => $a.primary);
                if(lPrimary != null){ 
                    console.log('primary action detected',lPrimary,$scope.actions);
                    return lPrimary.action();
                }
            }
        }

        if(lKeyMap[lKeyCode] == undefined) return;


        lKeyMap[lKeyCode]();
    }
})

id3Plugin
.controller('panelController', function($scope, $rootScope, $q, $timeout, $id3Lists, $params, mdPanelRef){
    $scope.panelOptions = {
        actions : []
    };

    $scope.$id3Lists = $id3Lists;

    $scope.cancel = function(){
        $scope.close();
    }
    $scope.close = function(){
        mdPanelRef.close();
    }

    $scope.closeAndReturn = function($value){
        mdPanelRef.close();
        mdPanelRef.callback($value)
    }

    $scope.$params = $params;
    
    $scope.panelInit = function(){
        mdPanelRef.__result = null;
        console.log('panelInit', mdPanelRef);
    }

    $scope.trapPanelKeydown = function($event){
        if($scope.panelOptions.actions.length == 0) return;
        console.log('panelController/trapPanelKeydown');
        const lKeyCode = $event.keyCode;
        const lKeyMap = {
            13 : _ => {
                const lPrimary = $scope.panelOptions.actions.find($a => $a.primary);
                if(lPrimary != null){ 
                    console.log('primary action detected',lPrimary,$scope.actions);
                    return lPrimary.action();
                }
            }
        }

        if(lKeyMap[lKeyCode] == undefined) return;


        lKeyMap[lKeyCode]();
    }
});

id3Plugin
.controller('wizardController', function($scope, $rootScope, $q, $element, $timeout, $id3Lists,$id3IO,$id3Tools, $wizardInfos, $params, mdPanelRef){
    $scope.wizardOptions = {
        actions : []
    };

    $scope.$actions = [];
    $scope.options = {
        showNavigation: true,
        showCancel: true,
        showPrevious: true,
        showNext: true,
        completeLabel: 'Wizard completed'
    }
    $scope.$params = $params;
    $scope.$id3Lists = $id3Lists;
    $scope.$wizard = $wizardInfos;
    $scope.$id3Tools = $id3Tools;
        
    $scope.wizardInit = function(){
        console.log('wizardInit', mdPanelRef);
        $scope.$wizard.currentIndex = 0;
        if($scope.$wizard.pages != undefined){
            $scope.registerPages($scope.$wizard.pages);
        }

        $scope.$on('id3Wizard/complete', function($event, $data, $options){
            console.log('@id3Wizard/complete',$data)
            $scope.exitWizard($data,$options);
        })
    }

    $scope.registerPages = function($pages){
        if(id3IsNullOrEmpty($pages)) return;

        console.log('wizardController/registerPages', $pages);

        $pages.forEach($p => {
            $p.path = $id3IO.absPath($p.path, 'views/statics/wizards/{0}.html',true);
        });
        $scope.$wizard.pages = $pages;

        $element[0].querySelector('.id3-wizard-container').classList.add('state-active');
    }

    $scope.registerPageAt = function($page, $index){
        if($scope.$wizard.pages.some($p => $p.key == $page.key)) return;
        $index = Math.min($index, $scope.$wizard.pages.length-1);

        $page.path = $id3IO.absPath($page.path, 'views/statics/wizards/{0}.html');
        
        $scope.$wizard.pages.splice($index,0, $page);
    }

    $scope.unregisterPages = function($pagekey_list){
        $scope.$wizard.pages = $scope.$wizard.pages.filter($p => !$pagekey_list.includes($p.key));
    }

    $scope.registerAction = function($key, $fn){
        $scope.$actions.push({
            key: $key, fn: $fn
        });
    }


    $scope.cancel = function(){
        $scope.close();
    }
    $scope.close = function(){
        mdPanelRef.close();
    }

    $scope.closeAndReturn = function($value){
        mdPanelRef.close();
        mdPanelRef.callback($value)
    }

    $scope.exitWizard = function($data, $options){
        $options = angular.merge({
            endAnimation : true
        },$options);

        if($options.endAnimation==false){
            $scope.closeAndReturn($data);
            return;
        }

        $scope.showEndAnimation().then(_ => {
            $scope.closeAndReturn($data);
        });
        
    }

    $scope.showEndAnimation = function(){
        $scope.$wizard.currentIndex++;
        const lCheckElm = $element[0].querySelector('id3-wizard-page svg .path.check');
        return $q( $resolve => {
            $timeout(_ => {
                lCheckElm.addEventListener('animationend', function($event){
                    $timeout(_ => {
                        $resolve();
                    },1000);
                });
            
                $element[0].querySelector('.id3-wizard-container').classList.add('state-complete');
            }, 500);
        })
        
    }

    $scope.trapWizardKeydown = function($event){
        console.log('wizardController/trapWizardKeydown');
        const lKeyCode = $event.key;
        if(!$event.ctrlKey) return;
        
        const lKeyMap = {
            'Escape' : _ => {
                // trigger cancel
                $scope.cancel();
            },
            'Backspace' : _ => {
                // trigger previous step
                if($scope.previousIsAvailable()){
                    $scope.previousStep();
                }
            },
            'Enter' : _ => {
                // trigger next step
                if($scope.nextIsAvailable()){
                    $scope.nextStep();
                }
            }
        }

        if(lKeyMap[lKeyCode] == undefined) return;

        lKeyMap[lKeyCode]();
    }

    $scope.registerValidation = function($key, $validations){
        const lPage = $scope.$wizard.pages.find($p => $p.key == $key);
        if(lPage.validations == undefined) lPage.validations = [];
        if(Array.isArray($validations)){
            lPage.validations = lPage.validations.concat($validations);
        }
        else{
            lPage.validations.push($validations);
        }
    }


    $scope.previousIsAvailable = function(){
        // there is no next page
        if($scope.$wizard.currentIndex == 0) return false;
        if($scope.$wizard.pages[$scope.$wizard.currentIndex] == undefined) return false;

        const lPage = $scope.$wizard.pages[$scope.$wizard.currentIndex];
        
        // there's no validation, so next page is available immediately
        if(lPage.validations === undefined) return true;
        
        // every validation must return true to allow next page
        const lResult = lPage.validations.every($v => {
            if(typeof $v.previous == 'function'){
                return $v.previous();
            }

            return true;
        });
        
        return lResult
    }

    $scope.nextIsAvailable = function(){
        // there is no next page
        if($scope.$wizard.pages[$scope.$wizard.currentIndex+1] == undefined) return false;
        const lPage = $scope.$wizard.pages[$scope.$wizard.currentIndex];

        // there's no validation, so next page is available immediately
        if(lPage.validations === undefined) return true;
        
        // every validation must return true to allow next page
        const lResult = lPage.validations.every($v => {
            if(typeof $v.next == 'function'){
                return $v.next();
            }

            return $v();
        });
        
        return lResult
    }
    $scope.completeIsAvailable = function(){
        if($scope.$wizard.pages[$scope.$wizard.currentIndex] == undefined) return false;

        const lPage = $scope.$wizard.pages[$scope.$wizard.currentIndex];

        // there's no validation, so complete is available immediately
        if(lPage.validations === undefined) return true;
        
        // every validation must return true to allow complete
        const lResult = lPage.validations.every($v => {
            if(typeof $v.next == 'function'){
                return $v.next();
            }

            return $v();
        });
        
        return lResult
    }

    $scope.isLastPage = function(){
        return($scope.$wizard.pages[$scope.$wizard.currentIndex+1] == undefined);
    }


    $scope.nextStep = function(){
        $scope.$broadcast('id3Wizard/pageLeave', $scope.$wizard.pages[$scope.$wizard.currentIndex]);
        
        $scope.$wizard.currentIndex = Math.min($scope.$wizard.currentIndex + 1, $scope.$wizard.pages.length - 1);
        $timeout( _ => {
            $scope.$broadcast('id3Wizard/pageEnter', $scope.$wizard.pages[$scope.$wizard.currentIndex]);
        },250);
    }

    $scope.previousStep = function(){
        $scope.$broadcast('id3Wizard/pageLeave', $scope.$wizard.pages[$scope.$wizard.currentIndex]);
        $scope.$wizard.currentIndex = Math.max($scope.$wizard.currentIndex -1, 0);
        $timeout( _ => {
            $scope.$broadcast('id3Wizard/pageEnter', $scope.$wizard.pages[$scope.$wizard.currentIndex]);
        },250);
    }

    $scope.complete = function(){
        if($scope.$actions.some($a => $a.key == 'buildResult')){
            $scope.$actions.find($a => $a.key == 'buildResult').fn().then( ($data,$options) => {
                console.log('id3Wizard/complete@', $data,$options);
                $rootScope.$broadcast('id3Wizard/complete', $data, $options);    
            });
        }
        else{
            $rootScope.$broadcast('id3Wizard/complete');
        }
    }

    $scope.setOptions = function($new){
        $scope.options = Object.assign($scope.options, $new);
    }
});


/**
 * Plugin main controller
 */

id3Plugin
.controller('frameworkCtrl', function($scope, $rootScope, $q, $timeout,$id3Settings, $id3Rest, $id3UI){
    $scope.isReady = false;
    $scope.settings = {
        color_palette: {
            primary: {h:0,s:0,l:100},
            secondary: {h:0,s:0,l:80},
            action: {h:38,s:100,l:50},
        },
        style_vars: {
            corner: {value: 0.25, unit: 'em'},
            spacing: {value: 1, unit: 'rem'},
            anim_speed: 0.5,
            normal_button: {
                roundness: 'max', // none/min/max
                style: 'hollow', // fill, hollow
                hover_animation: 'fade-in',
                icon: 'fal fa-arrow-right',
                icon_display:'none'
            },
            action_button: {
                roundness: 'default', // none/default/max
                style: 'fill', // fill, hollow
                hover_animation: 'fade-in',
                icon: '',
                icon_display:'none'
            }
        }
    };

    $scope.init = function(){
        $scope.pluginList = [];
        

        $rootScope.isReady()
            .then(_ => {
                console.log('Bootstrapper is ready to do stuff 2');
                
                return $id3Rest.call('framework/infos/plugins')
                .then($response=>{
                    $scope.pluginList = $response;
                });
            })
            .then(_ => {
                return $q.all([
                    $id3Settings.assignTo($scope,'networkSettings',true),
                    $id3Settings.load($scope,'settings')
                ])
            })
            .then(_ => {
                $scope.docs = {
                    'Getting started' : [
                        {path: 'getting-started/environment', name: 'Environment', description: 'All you need to begin your development journey'},
                        {path: '', name: 'Code norms', description: 'How to structure your code to comply to the Framework norms'},
                    ],
                    'Basics' : [
                        {path: '', name: 'Directives', description: 'Using the framework directives'},
                        {path: '', name: 'Filters', description: 'Using the framework filters'},
                        {path: '', name: 'Services', description: 'Using the framework factory services'},
                        {path: '', name: 'Utilities', description: 'Using the framework tools, classes and prototype functions'},
                        {path: '', name: 'Locales', description: 'Using the framework translation engine'},
                    ],
                    'PHP' : [
                        {path: '', name: 'Building your REST API', description: 'Extending the framework REST API to add your custom endpoints'}
                    ],
                    'Advanced' : [
                        {path: '', name: 'Dialogs', description: 'Creating your own dialog window'},
                        {path: '', name: 'Wizards', description: 'Creating your own wizards window'},
                    ]
                };
            })
            .then(_ => {
                // $id3UI.wizard('fw-setup').then(_ => {
                //     $id3UI.toast('Setup complete', "Don't forget to read the documentation")
                // });
                $scope.isReady = true;

            });
        
    }

    $scope.editColorPalette = function(){
        $id3UI.dialog('color-palette-editor', $scope.settings).then($newPalette => {
            $scope.settings.color_palette = $newPalette;
        });
    }

    $scope.editStyles = function(){
        $id3UI.dialog('styles-editor', $scope.settings).then($newStyles => {
            $scope.settings.style_vars = $newStyles;
        });
    }

    $scope.openDocumentation = function($item){
        if(id3IsNullOrEmpty($item.path)){ 
            console.log('no path for this doc');
            return ;
        }
        $id3UI.dialog($item.path);
    }

    $scope.showShortcodes = function(){
        $id3UI.dialog('shortcodes');
    }

    $scope.showRest = function(){
        $id3UI.dialog('rests');
    }

    
})

