id3Plugin
.filter('translate', function(){
    return function($str){
        if(id3IsNullOrEmpty($str)) return '';
        return $str.translate();
    }
});

id3Plugin
.filter('labelOf', ['$id3Lists', function($id3Lists){
    return $id3Lists.labelOf;
}]);

id3Plugin
.filter('attrOf', ['$id3Lists', function($id3Lists){
    return $id3Lists.attrOf;
}]);

id3Plugin
.filter('arrayJoin', [function(){
    return function($values, $glue=', ', $objectAttr=null){
        if($values == undefined) return '';
        if($values == null) return '';
        if(!Array.isArray($values)) return '';
        if($values.length == 0) return '';

        if($objectAttr != null && $values[0][$objectAttr] != undefined){
            return $values.map($v => $v[$objectAttr]).join($glue)
        }
        
        return $values.join($glue);
    }
}]);

id3Plugin
.filter('id3FileSize', [function(){
    return function($value){
        const sizeSuffix = ['bytes','Kb','Mb','Gb','Tb'];
        let lLoopCount = 0;
        while ($value > 1024) {
            $value = $value / 1024;
            lLoopCount++;
        }

        let lRoundValue = Math.round($value *  100) / 100;
        if(lRoundValue == Math.floor(lRoundValue)) lRoundValue = Math.round(lRoundValue);

        return lRoundValue.toString() + sizeSuffix[lLoopCount].translate();
    }
}])

id3Plugin
.filter('id3HslPart', [function(){
    return function($value, $parts='hsl'){
        const partArr = $parts.split('');
        const result  = [];
        if(partArr.includes('h') && $value.h != undefined) result.push($value.h);
        if(partArr.includes('s') && $value.s != undefined) result.push($value.s + '%');
        if(partArr.includes('l') && $value.l != undefined) result.push($value.l + '%');

        return result.join(', ');
    }
}]);

id3Plugin
.filter('id3CornerRoundness', [function(){
    return function($value){
        const roundnessMap = {
            'max' : '--id3-ui-corner: 2em',
            'none' : '--id3-ui-corner: 0'
        };

        if(roundnessMap[$value] == undefined) return '';
        return roundnessMap[$value];
        
    }
}])

id3Plugin
.filter('id3ColorPaletteStyle', function(){
    return function($palette, $baseName='id3-color'){
        const result = ['primary','secondary','action'].map($k => {
            const pal = $palette[$k];
            return `--${$baseName}-${$k}: hsl(${pal.h},${pal.s}%,${pal.l}%)`;
        });

        return result.join(';');
    }
})

id3Plugin
.filter('ifEmpty', [function(){
    return function($value, $default=''){
        if(id3IsNullOrEmpty($value)) return $default;
        return $value;
    }
}]);

id3Plugin
.filter('secretify', [function(){
    return function($value, $displayCharCount=5,$displaySide='both'){
        if($value == undefined) return '';
        
        const lValueLength = $value.toString().length;
        $displayCharCount = Math.min(lValueLength, $displayCharCount);

        const lChars = Array.from(Array(7)).map($e => '*').join('');
        const lStart = ['start','both'].includes($displaySide) ? $value.toString().substr(0,$displayCharCount) : '';
        const lEnd = ['end','both'].includes($displaySide) ? $value.toString().substr($displayCharCount*-1) : '';
        return lStart + lChars + lEnd;
    }
}]);
