class FontAwesomeQL {
    constructor({apiKey,kitName}={}){
        this.latest     = '6.1.1';
        this.apiKey     = apiKey;
        this.kitName    = kitName;
        this.token      = null;
    }

    static latestVersion(){
        return FontAwesomeQL.versions().then( versions => {
            return versions.pop();
        })
    }
    static versions(){
        const instance = new FontAwesomeQL();
        return instance.get(`{ releases { version } }`).then( $result => {
            return $result.releases;
        });
    }

    kits(){
        return this.get('{ me { kits { name } } }');
    }

    getToken(){
        if(this.apiKey == undefined) return Promise.resolve(null);
        if(this.token == null) return this.refreshToken();
        if(this.token.expire_date < new Date()) return this.refreshToken();

        return Promise.resolve(this.token);
    }

    refreshToken(){
        const path = 'https://api.fontawesome.com/token';
        const fetchOptions = {
            method: 'POST',
            headers: {
                Authorization : 'Bearer ' + this.apiKey
            },
        }

        return fetch(path, fetchOptions)
                .then(response => {
                    if(response.status == 200){
                        return response.json();
                    }

                    return response.text();
                })
                .then(result => {
                    //console.log('fetch2', result);
                    if(result.data != undefined) return result.data;
                    return result;
                })
                .then( token => {
                    this.token = token;
                    if(token != null){
                        const expireDate = new Date();
                        expireDate.setSeconds(expireDate.getSeconds() + token.expires_in);
                        this.token.expire_date = expireDate;
                    }
                    console.log('return token', this.token);
                    return this.token;
                });
    }

    search(query, {version=null, max=15} ){
        if(version == null) version = this.latest;

        return this.get(`{ search(version: "${version}", query: "${query}", first: ${(max + 1)}) { id, styles } }`).then( $result => {
            return $result.search;
        })
    }

    get(query, path='/'){
        const data = JSON.stringify({
            query: query
        });

        const hostname = 'https://api.fontawesome.com';
        path = hostname + path;
        
        const fetchOptions = {
            body: data,
            //port: 443,
            method: 'POST',
            headers: {
                'Content-Type': 'application/json',
                'Content-Length': data.length,
                //'User-Agent': 'Node',
            },
        }

        return this.getToken().then( token => {
            if(token != null){
                fetchOptions.headers.Authorization = `${token.token_type} ${token.access_token}`;
            }
    
            return fetch(path, fetchOptions)
                    .then(response => {
                        if(response.status == 200){
                            return response.json();
                        }
    
                        return response.text();
                    })
                    .then( result => {
                        if(result.data != undefined) return result.data;
                        return result;
                    })
        })
        
    }
}