id3Plugin
.factory('$id3IO', [
'$rootScope','$q','$http',
function($rootScope,$q,$http){
    const $scope = {};
    
    $scope.absPath = function($relPath, $format='{0}',$nocache=false){
        const lPath = $relPath.replace(/~\//g,'');

        let lAbsPath = '';
            
        if($relPath.match(/^~.+/) != null) {
            lAbsPath = '/' + $id3FrameworkContext.base_url + $format.format(lPath);
        }
        else{
            lAbsPath = '/' + $id3PluginContext.base_url + $format.format(lPath);
        }

        if($nocache){
            lAbsPath = lAbsPath + '?t=' + (new Date()).getTime();
        }

        return lAbsPath;
    }

    $scope.toClipboard = function($data){
        let lContent = '';
        if($data !== null && typeof $data === 'object'){
        lContent = JSON.stringify($data);
        }
        else{
        lContent = $data;
        }

        let lTextarea = document.createElement("textarea");
        lTextarea.textContent = lContent;
        lTextarea.style.position = "fixed";  // Prevent scrolling to bottom of page in MS Edge.
        document.body.appendChild(lTextarea);
        lTextarea.select();
        try {
            
            document.execCommand("copy");  // Security exception may be thrown by some browsers.
        } 
        catch (ex) {
            console.warn("Copy to clipboard failed.", ex);
            return false;
        } 
        finally {
            document.body.removeChild(lTextarea);
            $rootScope.$broadcast('$id3UI/call','toast','Copied to clipboard');
        }
    }

    $scope.addScript = function($key, $path){
        const lElementId = `${$key}-script`
        const lDeferred = $q.defer();

        if(document.body.querySelector('#' + lElementId) == null){
            const lScriptElement = document.createElement('script');
            lScriptElement.src = $path;
            lScriptElement.id = lElementId;

            lScriptElement.addEventListener('load', _ => {
                lDeferred.resolve();
            });
            document.body.append(lScriptElement);
        }
        else{
            return $q.resolve();
        }

        return lDeferred.promise;
    }

    $scope.addStyle = function($key, $path){
        const lElementId = `${$key}-style`
        const lDeferred = $q.defer();

        if(document.body.querySelector('#' + lElementId) == null){
            const lStyleElement = document.createElement('link');
            lStyleElement.href = $path;
            lStyleElement.rel = 'stylesheet';
            lStyleElement.id = lElementId;

            lStyleElement.addEventListener('load', _ => {
                lDeferred.resolve();
            });
            document.body.append(lStyleElement);
        }
        else{
            return $q.resolve();
        }

        return lDeferred.promise;
    }


    $scope.fileSelect = function(fileTypes=null){
        const lFileInput = document.createElement('input');
        const lDeferred = $q.defer();

        lFileInput.setAttribute('type','file');
        if(fileTypes != null){
            lFileInput.setAttribute('accept', fileTypes);
        }
        lFileInput.style.opacity = 0;
        lFileInput.style.position = 'absolute';
        lFileInput.style.left = '-20vw';

        lFileInput.addEventListener('change',function($event){
            console.log('selectFile@input_change');

            if(this.files.length > 0){
                const lResult = this.files[0];

                lDeferred.resolve(lResult);
            }

            lFileInput.remove();
        });

        window.addEventListener('focus', () => {
            setTimeout(() => {
                // remove dom
                if(lFileInput!=null){
                    lFileInput.remove();
                }
                
                lDeferred.reject();
            }, 300)
        }, { once: true })

        document.body.appendChild(lFileInput);

        lFileInput.click();

        return lDeferred.promise;
    }

    $scope.fileGetArrayBuffer = function($file){
        const lDeferred = $q.defer();
        var reader = new FileReader();

        reader.addEventListener('load', function($event){
            lDeferred.resolve(this.result);
        });

        reader.readAsArrayBuffer($file);

        return lDeferred.promise;
    }

    $scope.fileGetTextContent = function($file){
        const lDeferred = $q.defer();
        var reader = new FileReader();

        reader.addEventListener('load', function($event){
            lDeferred.resolve(this.result);
        });

        reader.readAsText($file);

        return lDeferred.promise;
    }

    $scope.fileGetUrlContent = function($file){
        const lDeferred = $q.defer();
        var reader = new FileReader();

        reader.addEventListener('load', function($event){
            lDeferred.resolve(this.result);
        });

        reader.addEventListener('error', function($event){
            lDeferred.reject();
        });

        reader.readAsDataURL($file);

        return lDeferred.promise;
    }

    $scope.fileGetImageSize = function($file){
        const img = document.createElement("img");

        const promise = new Promise((resolve, reject) => {
            img.onload = () => {
                // Natural size is the actual image size regardless of rendering.
                // The 'normal' `width`/`height` are for the **rendered** size.
                const width  = img.naturalWidth;
                const height = img.naturalHeight; 

                // Resolve promise with the width and height
                resolve({width, height});
            };

            // Reject promise on error
            img.onerror = reject;
        });

        // Setting the source makes it start downloading and eventually call `onload`
        if(typeof $file == 'string'){
            img.src = url;
        }
        else{
            $scope.fileGetUrlContent($file).then( $url => {
                img.src = $url;
            });
        }
        

        return promise;
    }

    $scope.getData = function($filename){
        const lFilepath = $scope.absPath('scripts/data/' + $filename);
        return $http({url: lFilepath}).then($response => {
            if($response.status == 200){
                return $response.data;
            }
            else{
                return null;
            }
        })
    }


    return $scope;
}]);

id3Plugin
.factory('$id3Tools', [
'$rootScope',
function($rootScope){
    const $scope = {};
    
    $scope.toggle = function($object, $attr, $list=null){
        if($list != null && Array.isArray($list)){

            $list.forEach($e => {
                if($object.$$hashKey && $e.$$hashKey==$object.$$hashKey) return;
                delete $e[$attr];
            });
        }
        console.log('$id3Tools/toggle', $object, $attr, $list);
        $object[$attr] = ($object[$attr] == undefined || $object[$attr]==null || $object[$attr]==false) ? true : false;
    }

    return $scope;
}]);

id3Plugin
.factory('$id3UI', [
'$rootScope', '$compile', '$timeout', '$q','$mdDialog', '$mdPanel', '$mdToast',
function id3UI($rootScope,$compile, $timeout, $q, $mdDialog, $mdPanel, $mdToast){
    const $scope = {};

    $scope.isReady = function(){
        $rootScope.$on('$id3UI/call', function($event, $method,$params){
            if(!Array.isArray($params)) $params = [$params];
            if(typeof $scope[$method] == 'function'){
                $scope[$method](...$params);
            }
        });

        return $q.resolve();
    }

    $scope.__longProcessHndl = null;
    $scope.startLongProcess = function($start_timeout=500){
        if($scope.__longProcessHndl != null){
            $timeout.cancel($scope.__longProcessHndl);
        }
        
        if($start_timeout <= 0) return;

        console.log('Showing long process panel in', $start_timeout, 'ms');

        $scope.__prepareLongProcessPanel();
        
        $scope.__longProcessHndl = $timeout(_ => {
            $scope._showLongProcessPanel();
        }, $start_timeout);
    }

    $scope.__prepareLongProcessPanel = function(){
        let lPanel = document.querySelector('#id3-ui-long-process-panel');
        if(lPanel == null){
            lPanelTemplate = `<div id="id3-ui-long-process-panel">
                <id3-svg id3-src="~/assets/spinner"></id3-svg>
                <label ls>Working hard, please wait</label>
            </div>`;
            lAngPanel = angular.element(lPanelTemplate)
            $compile(lAngPanel)($rootScope);
            lPanel = lAngPanel[0];
            document.body.append(lPanel);
        }
        return lPanel;
    }
    $scope._showLongProcessPanel = function(){
        const lPanel = $scope.__prepareLongProcessPanel();
        lPanel.classList.add('show');
    }

    $scope.stopLongProcess = function(){
        if($scope.__longProcessHndl != null){
            $timeout.cancel($scope.__longProcessHndl);
        }
        $scope._hideLongProcessPanel();

    }
    $scope._hideLongProcessPanel = function(){
        const lPanel = document.querySelector('#id3-ui-long-process-panel');
        if(lPanel != null){
            lPanel.classList.remove('show');
        }
    }

    $scope.alert = function($title, $text='', $options=null){
        $options = angular.merge({
            ok: 'OK',
            ev: null
        },$options);

        const lDiag = $mdDialog.alert()
                        .parent(angular.element(document.body))
                        .clickOutsideToClose(true)
                        .title($title.translate())
                        .ok($options.ok.translate())
                        .targetEvent($options.ev);

        if($text != ''){
            lDiag.textContent($text.translate());
        }
                        
        return $mdDialog.show(lDiag);

    }
    $scope.sidePanel = function($template, $params, $options){
        const lOptions = Object.assign({
            animationFrom: 'right',
            animationTo: 'right',
            panelClass: 'id3-side-panel panel-right',
            position: $scope.getPanelPosition('right'),
            animationType: $mdPanel.animation.SLIDE,
        }, $options);

        if(lOptions.side == 'left'){
            lOptions.animationFrom = 'left';
            lOptions.animationTo = 'left';
            lOptions.panelClass = 'id3-side-panel panel-left';
            lOptions.position = $scope.getPanelPosition('center');
        }

        return $scope.panel($template, $params, lOptions);
    }
    $scope.progressPanel = function($promiseList, $options){
        const lPanelPosition = {
            x: window.innerWidth / 2,
            y: window.innerHeight
        }
        const panelOptions = {
            clickOutsideToClose: false,
            clickEscapeToClose: false,
            animation: $mdPanel.newPanelAnimation()                           
                                    .duration(500)
                                    .withAnimation('animation-none')
        }

        return $scope.panel('~/progress-panel', {steps: $promiseList, options: $options},panelOptions);
    }

    $scope.panel = function($template, $params, $options){
        const lOptions = Object.assign({
            //parent: angular.element(document.body),
            //animation: animation,
            attachTo: angular.element(document.body),
            // controller: DialogCtrl,
            // controllerAs: 'ctrl',
            //templateUrl: 'panel.tmpl.html',
            panelClass: 'id3-panel',
            position: $scope.getPanelPosition('center'),
            trapFocus: true,
            zIndex: 150,
            clickOutsideToClose: true,
            clickEscapeToClose: true,
            hasBackdrop: true,
            animationType: $mdPanel.animation.SCALE,
            animationDuration: 250,
            animationFrom: 'center',
            animationTo: 'center',
            locals : {
              $params: $params
            }
          },$options
        );
        
        console.log('$id3UI/panel',$template);

        if(lOptions.animation == undefined){
            lOptions.animation = $mdPanel.newPanelAnimation()
                                    .openFrom($scope.getPanelPosition(lOptions.animationFrom))
                                    .duration(lOptions.animationDuration)
                                    .closeTo($scope.getPanelPosition(lOptions.animationTo))
                                    .withAnimation(lOptions.animationType);
        }
        
        console.log('$id3UI/panel',$template, lOptions);

        const lTemplateFileName = $template.replace(/~\//g,'');
        let lTemplateUrl = '';
        
        if($template.match(/^~.+/) != null) {
            lTemplateUrl = '/' + $id3FrameworkContext.base_url + 'views/statics/panels/' + lTemplateFileName + '.html';
        }
        else{
            lTemplateUrl = '/' + $id3PluginContext.base_url + 'views/statics/panels/' + $template + '.html';
        }
        
        
        // detect controller from filename
        if(lOptions.controller == undefined){
            const lCtrlName = lTemplateFileName.toCamelCase() + 'Ctrl';
        
            if(typeof window[lCtrlName] == 'function'){
                console.log('assign root base function as dialog controller');
                lOptions.controller = window[lCtrlName];
                lOptions.template = '<md-dialog class="id3-plugin-element" ng-init="init()"><ng-include src="\'' + lTemplateUrl + '\'"></ng-include></md-dialog>';
            }
            else{
                const lCtrlProvider = id3Plugin._invokeQueue
                                        .filter($p => $p[0] == "$controllerProvider")
                                        .find($p => $p[2][0] == lCtrlName);
                console.log('$id3UI/panel',lCtrlName, id3Plugin._invokeQueue.filter($p => $p[0] == "$controllerProvider"))
               
                if(lCtrlProvider != null){
                    console.log('$id3UI/panel', lCtrlProvider[2][1])
                    lOptions.controller = 'panelController';
                }

                lOptions.template = '<div class="id3-plugin-element" id="' + lTemplateFileName  + '" ng-init="panelInit()" ng-keydown="trapPanelKeydown($event)"><div ng-controller="' + lCtrlName + '"  ng-init="init($params)"><ng-include src="\'' + lTemplateUrl + '\'"></ng-include></div></div>';
            }
            
            
        }
    
        return $q( $resolve => {
            $mdPanel.open(lOptions).then($panelRef => {
                $panelRef.callback = function($result){
                    $resolve($result);
                }

                // $panelRef.registerInterceptor($mdPanel.interceptorTypes.CLOSE, function($result){
                //     if(!id3IsNullOrUndefined($result.__result)){
                //         $resolve($result.__result);
                //     }
                // })
            });
        })
        
    }

    $scope.getPanelPosition = function($position){
        const lPositionMap = {
            'left'  : _ => $mdPanel.newPanelPosition()
                                .absolute()
                                .top()
                                .left(),
            'right' : _ => $mdPanel.newPanelPosition()
                                .absolute()
                                .top()
                                .right(),
            'center' : _ => $mdPanel.newPanelPosition()
                                .absolute()
                                .top('50%')
                                .right('50%')
        };
        console.log('$id3UI/getPanelPosition', lPositionMap[$position]());
        return lPositionMap[$position]();
    }


    $scope.wizard = function($name, $pages, $params, $options){
        $name = $name.toLowerCase();

        const lOptions = Object.assign({
            attachTo: angular.element(document.body),
            panelClass: 'id3-wizard',
            trapFocus: true,
            zIndex: 150,
            clickOutsideToClose: false,
            clickEscapeToClose: false,
            hasBackdrop: true,
            locals : {
              $wizardInfos: {
                  pages: $pages
              },
              $params: $params
            }
          },$options
        );
        
        
        // detect controller from filename
        if(lOptions.controller == undefined){
            const lCtrlName = $name.toCamelCase() + 'WizardCtrl';
    
        
            const lCtrlProvider = id3Plugin._invokeQueue
                                    .filter($p => $p[0] == "$controllerProvider")
                                    .find($p => $p[2][0] == lCtrlName);
            
            if(lCtrlProvider != null){
                lOptions.controller = 'wizardController';
            }
            else{
                console.log(lCtrlName, 'does not exists');
                return $q.reject();
            }
            
            lOptions.template = [
                '<div class="id3-plugin-element id3-wizard-container" id="' + $name  + '" ng-init="wizardInit()"  ng-keydown="trapWizardKeydown($event)">',
                    '<div ng-controller="' + lCtrlName + '"  ng-init="init($params)">',
                        '<div class="id3-wizard-progress" ng-if="$wizard.currentIndex<=$wizard.pages.length"><div class="id3-wizard-page"><sup>{{$wizard.currentIndex  + 1}}</sup><sub>{{$wizard.pages.length}}</sub></div></div>',
                        '<div class="id3-wizard-page-container">',
                            '<div class="id3-wizard-page-list" ng-if="$wizard.pages != null" style="--current-page: {{$wizard.currentIndex}}" id3-key-scope >',
                                '<ng-include ng-repeat="page in $wizard.pages" src="page.path"></ng-include>',
                                '<id3-wizard-page class="id3-page-complete">',
                                    '<id3-svg id3-src="~/assets/icons/check"></id3-svg>',
                                    '<h2>{{options.completeLabel.translate()}}</h2>',
                                '</id3-wizard-page>',
                            '</div>',
                        '</div>',
                        '<div class="id3-wizard-navigation" ng-if="options.showNavigation">',
                            '<md-button ng-if="options.showCancel" ng-click="cancel()"><i class="fal fa-times"></i> <ls>Cancel</ls></md-button>',
                            '<md-button ng-if="options.showPrevious" ng-disabled="!previousIsAvailable()" ng-click="previousStep()" class="md-icon-button"><i class="fal fa-arrow-left"></i></md-button>',
                            '<md-button ng-if="options.showNext && !isLastPage()" ng-disabled="!nextIsAvailable()" ng-click="nextStep()" class="md-primary md-raised animate__animated animate__fadeInUp"><ls>Next</ls> <i class="fal fa-arrow-right"></i></md-button>',
                            '<md-button ng-if="options.showNext && isLastPage()" ng-disabled="!completeIsAvailable()" ng-click="complete()" class="md-primary md-raised animate__animated animate__fadeInUp"><ls>Finish</ls> <i class="fal fa-check"></i></md-button>',
                        '</div>',
                    '</div>',
                '</div>'
            ].join('');            
        }
    
        console.log('$id3UI/wizard',$name, lOptions);

        return $q( $resolve => {
            $mdPanel.open(lOptions).then($panelRef => {
                $panelRef.callback = function($result){
                    $resolve($result);
                }
            });
        })


    }

    $scope.confirm = function($message, $hint='',$options=null){
        $options = angular.merge({
            ok: 'Yes',
            cancel: 'No',
            ev: null
        },$options);

        const lDiag = $mdDialog.confirm()
                        .parent(angular.element(document.body))
                        .clickOutsideToClose(true)
                        .title($message.translate())
                        .ok($options.ok.translate())
                        .cancel($options.cancel.translate())
                        .targetEvent($options.ev);

        if($hint != ''){
            lDiag.textContent($hint.translate());
        }

        return $mdDialog.show(
            lDiag
          );
    }

    /**
     * Call a dialog to open
     * @param {string} $template Url or String containing the template 
     * @param {*} $params parameters to pass to the dialog
     * @param {*} $options options for the dialog
     */
    $scope.dialog = function($template, $params=null, $options=null){
        const lOptions = Object.assign({
                parent: angular.element(document.body),
                targetEvent: null,
                clickOutsideToClose:true,
                fullscreen: true,
                multiple:true,
                locals : {
                    $params: $params
                }
            },
            $options
        );
        
        // console.log($template);
        const lTemplateFileName = $template.replace(/~\//g,'');
        let lTemplateUrl = '';
        
        if($template.match(/^~.+/) != null) {
            lTemplateUrl = '/' + $id3FrameworkContext.base_url + 'views/statics/dialogs/' + lTemplateFileName + '.html';
        }
        else{
            lTemplateUrl = '/' + $id3PluginContext.base_url + 'views/statics/dialogs/' + $template + '.html';
        }
        lTemplateUrl = lTemplateUrl + '?v=' + (new Date()).getTime();
        if(lOptions.zIndex != undefined){
            lOptions.onShowing = ($s, $e, $o, $c) => {
                $e[0].style.zIndex = lOptions.zIndex;
            }
        }
        // detect controller from filename
        if(lOptions.controller == undefined){
            
            let lCtrlName = lTemplateFileName.toCamelCase() + 'DialogCtrl';
            const lCtrlAltName =  lTemplateFileName.toCamelCase() + 'Ctrl';
            //console.log('search for controller', lCtrlName, typeof window[lCtrlName]);

            if(typeof window[lCtrlName] == 'function'){
                // console.log('assign root base function as dialog controller');

                lOptions.controller = window[lCtrlName];
                lOptions.template = '<md-dialog class="id3-plugin-element" ng-init="init()"><ng-include src="\'' + lTemplateUrl + '\'"></ng-include></md-dialog>';
            }
            else{
                const lCtrlProviders = id3Plugin._invokeQueue
                                        .filter($p => $p[0] == "$controllerProvider")
                                        .filter($p => [lCtrlName,lCtrlAltName].includes($p[2][0]));

                // console.log('$id3UI/dialog',lCtrlName, id3Plugin._invokeQueue.filter($p => $p[0] == "$controllerProvider"));

                lOptions.controller = 'dialogController';
                let lDialogSubController = '';
                if(lCtrlProviders != null && lCtrlProviders.length > 0) {
                    if(lCtrlAltName == lCtrlProviders[0][2][0]){
                        console.warn('$id3UI/dialog: Controller name does not respect naming convention.',lCtrlName , ' must be use instead of', lCtrlAltName);
                        lCtrlName = lCtrlAltName;
                    }
                
                    //console.log('$id3UI/dialog', lCtrlProviders[0][2][0])
                    
                    lDialogSubController = 'ng-controller="' + lCtrlName + '" ng-init="init($params)"';
                }

                lOptions.template = '<md-dialog class="id3-plugin-element" id="' + lTemplateFileName  + '" ng-init="dialogInit()" ng-keydown="trapDialogKeydown($event)"><div ' + lDialogSubController + '"><ng-include src="\'' + lTemplateUrl + '\'"></ng-include></div></md-dialog>';
            }
            
        }

        console.log('$id3UI/dialog:options', lOptions);

        return $mdDialog.show(lOptions);
    }

    $scope.toast = function($message, $options=null){
        $options = angular.merge({
            position: 'bottom right',
            delay: 3000
        },$options);

        return $mdToast.show(
            $mdToast.simple()
                .toastClass('id3-toast')
                .textContent($message.translate())
                .position($options.position)
                .hideDelay($options.delay)
        );
    }

    $scope.waitFor = function($elementSelector, $parent=null){

        $parent = ($parent == null) ? document.body : $parent;

        return $q( ($resolve, $reject) => {
            const lElement = $parent.querySelector($elementSelector);
        
            console.log('$id3UI/waitFor', lElement);
            if(lElement) {
              $resolve(lElement);
              return;
            }
        
            const lObserver = new MutationObserver(function(mutations) {
                console.log('$id3UI/waitFor/mutation/observed', mutations);
                const lExpectedElementCount = $elementSelector.split(',').length;
                const lSearchElements = $parent.querySelectorAll($elementSelector);
                if(lSearchElements != null && lSearchElements.length >= lExpectedElementCount ){
                    lObserver.disconnect();
                    $resolve(lSearchElements);
                    return;
                }
            });
        
            lObserver.observe($parent, { childList: true, subtree: true });
        });
    }

    

    return $scope;
}]);

id3Plugin
.factory('$id3Rest', [
'$rootScope', '$timeout', '$q', '$http', '$id3UI',
function id3Rest($rootScope, $timeout, $q, $http, $id3UI){
    const $scope = {
        basePath: null
    };

    $scope.isReady = function(){
        return $q.resolve();
    }

    $scope.wp = function($path, $data, $options){
        console.log('id3Rest/wp', $path, $data, $options);
        $options = angular.merge({
            url     : '/wp-json/wp/v2/' + $path,
            method  : typeof($data)=='undefined' ? 'GET' : 'POST',        
            headers: {
                'X-WP-Nonce': $id3PluginContext.nonce
            },
        }, $options);
    
        if($options.method=='GET'){
            $options.params = $data;
        }
        else{
            $options.data = $data;
        }
    
        // Setup promise object
        return $q(function($resolve, $reject){
            $http($options).then(
                // On success
                function success($result){
                if($result.status=='200'){
                    $resolve($result.data);
                }
                else{
                    $reject(null);
                }
                },
                // On fail
                function fail($error){
                    console.log('Fail on path', $path, 'with data' , $data , $error);
                }
            )
        });

    }

    $scope.call = function($path, $data, $options){
        console.log('$id3Rest/call', $path, $data, $options);
        $path = $path.replace(/^\/+|\/+$/g, '');
        const lBaseUrl = $scope.basePath != null ? $scope.basePath : $id3PluginContext.rest_root + 'id3-rest/';

        if(wpApiSettings && wpApiSettings.nonce != '' && (wpApiSettings.nonce != $id3PluginContext.nonce)){
            $id3PluginContext.nonce = wpApiSettings.nonce;
        }

        $options = angular.merge({
            backburner_timeout : 2000,
            url     : lBaseUrl + $path,
            method  : typeof($data)=='undefined' ? 'GET' : 'POST',        
            headers: ($scope.basePath == null) ? {
                'X-WP-Nonce': $id3PluginContext.nonce
            }: null,
        }, $options);
    

        if($options.method=='GET'){
            $options.params = $data;
        }
        else{
            $options.data = $data;
        }
        
        // Start long process UI backburner
        $id3UI.startLongProcess($options.backburner_timeout);

        // Setup promise object
        return $q(function($resolve, $reject){
            $http($options).then(
                // On success
                function success($result){
                    $id3UI.stopLongProcess();
                    if($result.status=='200'){
                        $resolve($result.data);
                    }
                    else{
                        $reject($result.data);
                    }
                },
                // On fail
                function fail($error){
                    $id3UI.stopLongProcess();
                    console.log('$id3Rest/call@fail to call', $path, 'with data' , $data , 'return: ', $error);
                    $reject($error);
                }
            );
            $scope.basePath = null;
        });
    }

    $scope.from = function($entrypoint){
        $scope.basePath = $entrypoint;
        return $scope;
    }

    $scope.get = function($path, $data){
        return $scope.call($path, $data, {method: 'GET'});
    }
    $scope.post = function($path, $data){
        return $scope.call($path, $data, {method: 'POST'});
    }
    $scope.delete = function($path, $data){
        return $scope.call($path, $data, {method: 'DELETE'});
    }
    
    return $scope;
}]);

id3Plugin
.factory('$id3Settings',[
'$rootScope', '$timeout', '$q', '$id3Rest', '$id3UI',
function id3Settings($rootScope, $timeout, $q, $id3Rest, $id3UI){
    const $scope = {
        $dataContainer : null,
        $dataAttr: 'data',
        $initial: null
    };
    
    $scope.isReady = function(){
        return $q.resolve();
    }

    $scope.assignTo = function($dataContainer, $attr, $network=false){
        const lRestEndpoint = ($id3PluginContext.is_network_admin || $network) ? 'settings/getNetwork' : 'settings/get';

        return $q( ($resolve,$reject) => {
            $id3Rest.call(lRestEndpoint, {plugin: $id3PluginContext.key},{method: 'GET'}).then($response => {
                $dataContainer[$attr] = $response;

                $resolve();
            })
        })
    }

    $scope.load = function($dataContainer, $attr, $network=false){

        if(id3IsNullOrEmpty($dataContainer)) throw('Must provide a data container');
        if($dataContainer.$watch == undefined) throw('Must provide a controller as data container');

        $scope.$dataContainer = $dataContainer;
        $scope.$dataAttr = $attr;
        $scope.clearWatch();
        
        $scope.$initial = angular.copy($dataContainer[$attr]);
        const lRestEndpoint = ($id3PluginContext.is_network_admin || $network) ? 'settings/getNetwork' : 'settings/get';

        return $q( ($resolve,$reject) => {
            $id3Rest.call(lRestEndpoint, {plugin: $id3PluginContext.key},{method: 'GET'}).then($response => {
                console.log('$id3Settings/load', $response);

                if($response != false){
                                        
                    Object.keys($scope.$dataContainer[$scope.$dataAttr]).forEach($k => {
                        if(!id3IsNullOrEmpty($response[$k])){
                            $scope.$dataContainer[$scope.$dataAttr][$k] = $response[$k];
                        }
                        else{
                            console.log($k, 'is null or empty in $response', $response[$k])
                        }
                    });
                    
                }

                $scope.applyWatch();
                
                $resolve($response);
            })
        })
    }

    $scope.clearWatch = function(){
        Object.keys($scope.$dataContainer[$scope.$dataAttr]).forEach($k => {
            if($scope.$dataContainer['$watch_' + $k] != undefined){
                $scope.$dataContainer['$watch_' + $k]();
            }
        });
    }
    $scope.applyWatch = function(){
        Object.keys($scope.$dataContainer[$scope.$dataAttr]).forEach($k => {
            console.log('$id3Settings/load/watching', $scope.$dataAttr + '.' + $k);
            $scope.$dataContainer['$watch_' + $k] = $scope.$dataContainer.$watch($scope.$dataAttr + '.' + $k, function($new, $old){
                        if($new != $old){
                            $scope.save($scope.$dataContainer[$scope.$dataAttr]);
                        }
                    },true);
        });
    }

    $scope.reset = function(){
        Object.keys($scope.$dataContainer[$scope.$dataAttr]).forEach($k => {
            $scope.$dataContainer[$scope.$dataAttr][$k] = $scope.$initial[$k];
        });
    }

    $scope.save = function($data=null){
        if($data == null) $data = $scope.$dataContainer[$scope.$dataAttr];
        const lRestEndpoint = ($id3PluginContext.is_network_admin) ? 'settings/saveNetwork' : 'settings/save';
        //console.log('$id3Settings/save', $data);
        return $id3Rest.call(lRestEndpoint, {plugin: $id3PluginContext.key, data: $data}).then(
            $response => {
                $id3UI.toast('Saved');
                return $response;
            },
            $fail => {
                $id3UI.alert('Unable to save. Check console for more details');
                console.log('$id3Settings/save@fail', $fail);
                return $fail;
            }
        );
    }

    return $scope;
}]);

id3Plugin
.factory('$id3Lists', [
'$rootScope', '$timeout', '$q',
function id3Lists(){
    const $scope = {
        _lists : {}
    };

    $scope.isReady = function(){
        return $q.resolve();
    }

    $scope.register = function($name, $values){
        $scope._lists[$name] = $values;
    }

    $scope.update = function($name, $values){
        $scope._lists[$name] = $values;
    }

    $scope.attrOf = function($key, $attr, $listName){
        if($scope._lists[$listName] == undefined) return $listName + '(NA)';
        const lResult = $scope._lists[$listName].find($e => $e.key == $key);
        if(lResult == null) return $key + '(NA)';
        if(lResult[$attr] == undefined) return $key + '.' + $attr + '(NA)'
        return lResult[$attr];
    }

    $scope.labelOf = function($key, $listName){
        if($scope._lists[$listName] == undefined) return $listName + '(NA)';
        const lValue = $scope._lists[$listName].find($e => $e.key == $key);
        if(lValue == null) return $key + '(NA)';

        return lValue.label.translate();
    }

    $scope.get = function($name){
        return $scope._lists[$name];
    }

    $scope.exists = function($item, $list){
        if($list == undefined) return false;
        if(!Array.isArray($list)) return false;

        return $list.includes($item.key)
    }

    $scope.toggle = function($item, $list){
        if($list == undefined) return;
        if(!Array.isArray($list)) $list = [];

        if(!$list.includes($item.key)){
            $list.push($item.key);
        }
        else{
            const lItemIndex = $list.findIndex($e => $e == $item.key);
            $list.splice(lItemIndex,1);
        }
    }

    $scope.getTree = function($name){

        const lBaseList = $scope._lists[$name];
        if(lBaseList == undefined) return null;

        const lFnGetChild = function($node){
            $node.children = lBaseList.filter($e => $e.parent == $node.key);
            $node.children.forEach(lFnGetChild);
        }

        const lTree = lBaseList.filter($e => $e.parent == undefined || $e.parent == null || $e.parent == 0);
        lTree.forEach(lFnGetChild);
        
        return lTree;

    }

    return $scope;
}]);

