

class ID3RestAPI{
    constructor($basePath=null){
        this._basePath = ($basePath != null) ? $basePath : '/wp-json/id3-rest/aliquando/';

        this._callStack = {};
    }

    static call($endpoint,$data=null,$options={}){
        if(window._instanceOfID3RestAPI == undefined){
            window._instanceOfID3RestAPI = new ID3RestAPI();
        }
        
        return window._instanceOfID3RestAPI.call($endpoint,$data,$options);
    }


    call($endpoint,$data=null,$options={}){
        let finalPath = this._buildEndpoint($endpoint);
        const url = new URL(finalPath);

        const fetchArgs =  Object.assign({
            method: 'GET'
        },$options);

        if($options.returnAs == undefined){
            $options.returnAs = 'json';
        }

        if($data != null){
            if($options.method == undefined){
                fetchArgs.method= 'POST';
            }

            if(fetchArgs.method === 'GET'){
                url.search = new URLSearchParams($data);
            }
            else{
                fetchArgs.data = $data;
            }
        }

        if(this._callStack[url.toString()] != undefined){
            const deferred = {};
            deferred.promise = new Promise(function(resolve, reject) {
                deferred.resolve = resolve;
                deferred.reject = reject;
            });

            this._callStack[url.toString()].push(deferred);
            return deferred.promise;
        }

        console.log('ID3RestApi/call', $endpoint, $options.returnAs);
        

        const promise = fetch(url, fetchArgs).then( $response => {
            if($response.status == 200){
                const lReturnWrapper = {
                    json : _ => $response.json(),
                    text: _ => $response.text(),
                    blob: _ => $response.blob()
                }

                const result = lReturnWrapper[$options.returnAs]();
                if(this._callStack[url.toString()] != undefined){
                   this._callStack[url.toString()].forEach($promise => {
                       $promise.resolve(result);
                   })
                   delete this._callStack[url.toString()];
                }

                return result
            }
            else{
                console.error('Call to ' + finalPath + ' failed:', $response.status, $response.body);
            }
        });

       
        this._callStack[url.toString()] = [];
        //this._callStack[url.toString()].push(promise);
        
        
        return promise;
    }

    _buildEndpoint($endpoint){
        let finalPath = this._basePath + $endpoint.trimChar('/');
        if($endpoint.startsWith('~/')){
            finalPath = '/wp-json/id3-rest/' + $endpoint.replace('~/','');
        }
        finalPath = window.location.protocol + '//' + window.location.host + finalPath;
        return finalPath;
    }
}


class ID3UI {
    constructor(){

    }

    compile($element=null){
        $element = $element == null ? document.body : $element;
    
        Object.entries(id3App.modules).forEach( ([$key, $module]) => {
            $module.apply($element);
        });
    }
}

class ID3Http{
    constructor(){

    }

    get($url, $options={}){
        const fetchArgs =  Object.assign({
            method: 'GET',            
        },$options);

        const url = new URL($url);

        if($options.returnAs == undefined){
            $options.returnAs = 'text';
        }
        
        return fetch(url, fetchArgs).then( $response => {
            

            if($response.status == 200){
                const lReturnWrapper = {
                    json : _ => $response.json(),
                    text: _ => $response.text(),
                    blob: _ => $response.blob()
                }
                
                return lReturnWrapper[$options.returnAs]();
            }
            else{
                console.error('Call to ' + $url + ' failed:', $response.status, $response.body);
            }
        });
    }
}



/**
 * @name ID3AppModule
 * @description Basic Shortcode class binder to init on window.load event
 */
 
const 
 STRIP_COMMENTS = /((\/\/.*$)|(\/\*[\s\S]*?\*\/))/mg,
 ARGUMENT_NAMES = /([^\s,]+)/g;

class ID3AppModule{

    constructor($moduleName){
        this.settings = null;
        this.directives = [];
        
        this.moduleName = $moduleName!=undefined ? $moduleName : this.constructor.name;
        id3App.addModule(this.moduleName, this);

        document.addEventListener('DOMContentLoaded', $event => {
            if(this.init != undefined){
                const injection = {
                    $restAPI: new ID3RestAPI(),
                    $http: new ID3Http(),
                    $ui: new ID3UI(),
                };

                const lParams = this._getParamNames(this.init);
                this.init.call(this,...lParams.filter($p => injection[$p]!=undefined).map($p => injection[$p]));
            }

            this.apply();
        });
    }

    addDirective($elementOrDirective, $directiveOptions){
        if($elementOrDirective instanceof ID3Directive){
            this.directives.push($elementOrDirective);
        }
        else{
            const directive = new ID3Directive($elementOrDirective, $directiveOptions);
            this.directives.push(directive);
        }
    }

    apply($scopeElement){
        //console.log('ID3AppModule/apply',this.moduleName,this.directives, $scopeElement);
        this.directives.forEach($directive =>{
            $directive.apply($scopeElement)
        });
    }

    getSettings(){
        if(this.settings != null) return Promise.resolve(this.settings);
        if(id3 != undefined && id3.settings[this.moduleName] != undefined) return Promise.resolve(id3.settings[this.moduleName]);
        
        return new Promise( ($resolve, $reject) => {
            ID3RestAPI.call('~/settings/get',{plugin: this.moduleName},{method: 'GET'}).then( $response => {
                this.settings = $response;
                $resolve(this.settings);
            });
        });
    }

    _getParamNames(func) {
        const fnStr = func.toString().replace(STRIP_COMMENTS, '');
        let result = fnStr.slice(fnStr.indexOf('(')+1, fnStr.indexOf(')')).match(ARGUMENT_NAMES);
        if(result === null)
            result = [];
        return result;
    }

}

class ID3Directive{
    constructor($target, $options){
        this.target = $target;
        this.options = Object.assign({
            restrict: 'C',
            onApply: function(){},
            controller: this
        },$options);
    }

    apply($applyScope){
        $applyScope = $applyScope == undefined ? document.body : $applyScope;
        const query = this.getTargetQuery();
        const targetElements = Array.from($applyScope.querySelectorAll(query));
        if($applyScope.matches(query)) targetElements.push($applyScope);
        
        targetElements
            .filter($elm => $elm._directive == undefined)
            .forEach($elm => {
                const injection = {
                    $restAPI: new ID3RestAPI(),
                    $http: new ID3Http(),
                    $ui: new ID3UI(),
                    $element: $elm,
                };

                const lParams = this._getParamNames(this.options.controller);
                const lController = this.options.controller;
                if(!(lController instanceof ID3Directive)){
                    lController.call(null,...lParams.filter($p => injection[$p]!=undefined).map($p => injection[$p]));    
                }
                
                this.addListeners($elm);
                
                this.options.onApply($elm);

                if(typeof lController.init == 'function'){
                    lController.init($elm, $elm.dataset);
                }

                $elm._directive = true;
            });
    }

    addListeners($element){
        ['click','mouseover','mouseout','focus','blur','swipeleft','swiperight','swipedown','swipeup'].forEach( $evtName => {
            $element.querySelectorAll('[id3-' + $evtName + ']').forEach($eventTarget => {

                const eventFn = $eventTarget.getAttribute('id3-' + $evtName);
                const eventFnName = eventFn.indexOf('(')>0 ? eventFn.substr(0,eventFn.indexOf('(')) : eventFn;

                if(this[eventFnName] == undefined){
                    console.log(eventFnName, 'function does not exists on ', this);
                }

                const evtHandler = ($event) => {
            
                    const lEvtAtt = $eventTarget.getAttribute('id3-' + $evtName);
                    if(lEvtAtt){
                        try {
                            eval(`this.${lEvtAtt}`);    
                        } catch (error) {
                            console.log('Error occured on ', $evtName, error);
                        }
                    }
                };
                
                if(['swipeleft','swiperight','swipedown','swipeup'].includes($evtName)){
                    const listener = new ID3SwipeHandler($eventTarget);
                    listener.on($evtName, evtHandler);
                }
                else{
                    $eventTarget.addEventListener($evtName, evtHandler,false);
                }
                
            })
        })

        
    }

    getTargetQuery(){
        const restrictions = this.options.restrict.toUpperCase().split('');
        const toDashString = this.target.replace(/([A-Z])/g, function($match){ return '-' + $match.toLowerCase()});
        const query = restrictions.map( $r => {
            return {
                '#' : '#' + toDashString,
                'C' : '.' + toDashString,
                'A' : '[' + toDashString + ']',
                'E' : toDashString
            }[$r];
        }).join(',');
        return query;
    }   
    
    _getParamNames(func) {
        const fnStr = func.toString().replace(STRIP_COMMENTS, '');
        let result = fnStr.slice(fnStr.indexOf('(')+1, fnStr.indexOf(')')).match(ARGUMENT_NAMES);
        if(result === null)
            result = [];
        return result;
    }
}

/**
 * @name ID3Shortcode
 * @description Basic Shortcode class binder to init on window.load event
 */
 class ID3Shortcode{
    constructor(){
        document.addEventListener('DOMContentLoaded', $event => {
            this.init();
        });
    }

    init(){}
}

class ID3Application{

    constructor(){
        this.modules = {};
    }

    addModule($name, $moduleInstance){
        this.modules[$name] = $moduleInstance;
    }
}
var id3App = new ID3Application();
