class id3ElementCollection extends Array{
    
    ready($callback){
        
        const isReady = this.some( $e => {
            return $e.readyState != null && $e.readyState != 'loading';
        });

        if(isReady){      
            $callback();
        }
        else{
            this.on('DOMContentLoaded',  $callback);
        }
    
    }

    on($eventName, $callbackOrSelector, $callback){
        if(typeof $callbackOrSelector === 'function'){
            this.forEach($e => $e.addEventListener($eventName, $callbackOrSelector));
        }
        else{
            this.forEach($e => {
                $e.addEventListener($eventName, $evt => {
                    if($evt.target.matches($callbackOrSelector)) $callback($evt);
                })
            })
        }
    }

    next(){
        return this.map($e => $e.nextElemenetSibling).filter($e => $e != null);
    }

    prev(){
        return this.map($e => $e.previousElemenetSibling).filter($e => $e != null);
    }

    removeClass($className){
        this.forEach($e => $e.classList.remove($className));
        return this;
    }
    addClass(...$className){

        this.forEach($e => $e.classList.add(...$className));
        return this;
    }
    
    append($tagOrElm){
        
        let lElm = $tagOrElm;
        if(typeof $tagOrElm === 'string' || $tagOrElm instanceof String) {
            if($tagOrElm.indexOf('<') < 0){
                lElm = document.createElement($tagOrElm);
            }
            else{
                const lContainer = document.createElement('div');
                lContainer.append($tagOrElm);
                lElm = lContainer.childNodes[0];
            }
        }
        this.forEach($e => $e.append(lElm));
        return id3Query(lElm);
    }

    toggleClass($className){
        this.forEach($e => $e.classList.toggle($className));
        return this;
    }
    css($propOrObject, $value){
        this.forEach($e => {
            if(typeof $propOrObject === 'string' || $propOrObject instanceof String){
                const lCamelProp = $propOrObject.replace(/(-[a-z])/g, $g => {
                    return $g.replace('-','').toUpperCase();
                });

                $e.style[lCamelProp] = $value;
            }
            else{
                Object.keys($propOrObject).map($prop => {
                    return $prop.replace(/(-[a-z])/g, $g => {
                        return $g.replace('-','').toUpperCase();
                    }); 
                }).forEach($prop => {
                    $e.style[$prop] = $value;
                });
            }
        });

        return this;
    }
}

function id3Query($param){
    if(typeof $param === 'string' || $param instanceof String){
        return new id3ElementCollection(...document.querySelectorAll($param));
    }
    else{
        return new id3ElementCollection($param);
    }
}

class ID3SwipeHandler{
    constructor($elm){
        this._actions = {};
        this._touchContext = {};
        this.allowedTime = 300;
        this.threshold = 150; //required min distance traveled to be considered swipe
        this.restraint = 100;

        $elm.addEventListener('touchstart', ($e) => this.handleStart($e),false);
        $elm.addEventListener('touchmove', ($e) => this.handleMove($e),false);
        $elm.addEventListener('touchend', ($e) => this.handleEnd($e),false);

    }


    on($eventName, $callback){
        if(this._actions[$eventName] == undefined) this._actions[$eventName] = [];
        this._actions[$eventName].push($callback);
    }

    handleStart($event){
        var touchobj = $event.changedTouches[0]
        this._touchContext.swipedir = 'none';
        this._touchContext.dist = 0;
        this._touchContext.startX = touchobj.pageX;
        this._touchContext.startY = touchobj.pageY;
        this._touchContext.startTime = new Date().getTime() // record time when finger first makes contact with surface

        $event.preventDefault()
    }
    handleMove($event){
        $event.preventDefault() // prevent scrolling when inside DIV
    }
    handleEnd($event){
        var touchobj = $event.changedTouches[0]
        const distX = touchobj.pageX - this._touchContext.startX // get horizontal dist traveled by finger while in contact with surface
        const distY = touchobj.pageY - this._touchContext.startY // get vertical dist traveled by finger while in contact with surface
        const elapsedTime = new Date().getTime() - this._touchContext.startTime // get time elapsed
        this._touchContext.swipedir = '';

        if (elapsedTime <= this.allowedTime){ // first condition for awipe met
            if (Math.abs(distX) >= this.threshold && Math.abs(distY) <= this.restraint){ // 2nd condition for horizontal swipe met
                this._touchContext.swipedir = (distX < 0)? 'left' : 'right' // if dist traveled is negative, it indicates left swipe
            }
            else if (Math.abs(distY) >= this.threshold && Math.abs(distX) <= this.restraint){ // 2nd condition for vertical swipe met
                this._touchContext.swipedir = (distY < 0)? 'up' : 'down' // if dist traveled is negative, it indicates up swipe
            }
        }

        this.trigger(this._touchContext.swipedir,$event);
        $event.preventDefault()
    }

    trigger(direction,event){
        
        if(this._actions.swipe != undefined) this._actions.swipe.forEach($fn => $fn(event,direction));
        if(this._actions[`swipe${direction}`] != undefined) this._actions[`swipe${direction}`].forEach($fn => $fn(event));
    }
}
